@@ -3,13 +3,12 @@ new_bench_expr <- function(x, description = names(x)) {
     description <- rep("", length(x))
   }
   names(x) <- description
-
-  structure(x, class = c("bench_expr", "list"))
+  structure(x, class = c("bench_expr", "list"), description = description)
 }
 
 #' @export
 format.bench_expr <- function(x, ...) {
-  desc <- names(x)
+  desc <- attr(x, "description")
   is_missing <- desc == ""
   desc[is_missing] <- vapply(x[is_missing], deparse_trunc, character(1))
   desc
@@ -24,6 +23,7 @@ print.bench_expr <- function(x, ...) {
   NextMethod()
 }
 
+#' @export
 type_sum.bench_expr <- function(x) {
   "bch:expr"
 }
@@ -34,8 +34,25 @@ type_sum.bench_expr <- function(x) {
   new_bench_expr(new_x)
 }
 
+# @export
+vec_proxy.bench_expr <- function(x, ...) {
+  desc <- attr(x, "description")
+  attributes(x) <- NULL
+  out <- list(x = x, desc = desc)
+  vctrs::new_data_frame(out, n = length(x))
+}
+
+# @export
+vec_restore.bench_expr <- function(x, to, ...) {
+  new_bench_expr(x$x, x$desc)
+}
+
+#' @export
 pillar_shaft.bench_expr <- function(x, ...) {
-  pillar::new_pillar_shaft_simple(format.bench_expr(x), align = "left", ...)
+  # We format bench expressions exactly like character vectors. This ensures
+  # they are truncated as needed, which is useful for long unnamed expressions
+  # (#94). This is the same logic as `pillar:::pillar_shaft.factor()`.
+  pillar_shaft(as.character(x), ...)
 }
 
 scale_type.bench_expr <- function(x) {
